from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.db import transaction
from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from .models import Payment
from quotations.models import Quotation, CustomerQuoteInfo
from account.models import Account

@login_required
def payments(request):
    if request.user.is_admin_user or request.user.is_super_user:
        payments = Payment.objects.all()
        return render(request, "sales/payment/index.html", {'payments': payments})

    if  request.user.is_sales: 
        payments = Payment.objects.filter(seller=request.user)
        return render(request, "sales/payment/index.html")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.", {'payments': payments})
    


@api_view(['POST']) 
def make_payment(request):
    try:
        # Extract data from request
        data = request.data
        transaction_id = data.get('transaction_id')
        amount_paid = data.get('amount_paid')
        quotation_id = data.get('quotation_id')
        payment_method = data.get('payment_method')  
        seller_id = data.get('seller_id')  
        request_user_id = data.get('request_user_id')  
        print(seller_id)
        print(request_user_id)

        # Validate the seller's permissions
        seller = get_object_or_404(Account, pk=seller_id)
        user = get_object_or_404(Account, pk=request_user_id)
        if not (user.is_superuser or user.is_admin_user or user.is_seller or user.is_pos):
            return Response({'error': 'Permission denied. You need admin, seller, or POS privileges to make payments.'}, status=status.HTTP_403_FORBIDDEN)
        
         # Validate if the transaction ID already exists
        if Payment.objects.filter(transaction_id=transaction_id).exists():
            return Response({'error': 'Transaction ID already exists. Please use a unique transaction ID.'}, status=status.HTTP_400_BAD_REQUEST)


        # Validate the quotation
        quotation = get_object_or_404(Quotation, pk=quotation_id)

        # Ensure quotation.amount_paid is not None
        if quotation.amount_paid is None:
            quotation.amount_paid = 0  # Assign a default value if it's None
 
        # Calculate total paid amount including the new payment
        total_paid = quotation.amount_paid + amount_paid

        # Check payment logic
        if payment_method == 'Full payment':
            if total_paid != quotation.total_amount:
                return Response({'error': 'Full payment requires the exact total amount to be paid.'}, status=status.HTTP_400_BAD_REQUEST)
        
        # Handle partial payments and overpayments
        if total_paid > quotation.total_amount:
            change = total_paid - quotation.total_amount
            amount_to_record = quotation.total_amount - quotation.amount_paid
            payment_status = 'PAID'
            
        else:
            change = 0
            amount_to_record = amount_paid
            payment_status = 'PARTIAL PAID' if total_paid < quotation.total_amount else 'PAID'

        # Record the payment
        with transaction.atomic():
            Payment.objects.create(
                quotation=quotation,
                payment_system_module='QUOTATION',  # Assuming payment system is always quotation
                transaction_id=transaction_id,
                amount_paid=amount_to_record,
                change=change,
                seller=seller
            )
            quotation.amount_paid += amount_to_record
            quotation.status = payment_status
            quotation.save()

        if payment_status == 'PAID':
            message = 'Payment processed successfully. The quotation is fully paid.'
        else:
            message = 'Payment processed successfully. The quotation is partially paid.'
        

        return Response({'message': message, 'change': change, 'status': quotation.status}, status=status.HTTP_201_CREATED)
    
    except KeyError as e:
        return Response({'error': f'Missing field: {str(e)}'}, status=status.HTTP_400_BAD_REQUEST)
    except Account.DoesNotExist:
        return Response({'error': 'Seller account does not exist.'}, status=status.HTTP_400_BAD_REQUEST)
    except Quotation.DoesNotExist:
        return Response({'error': 'Quotation does not exist.'}, status=status.HTTP_400_BAD_REQUEST)
    except Exception as e:
        print(f"Error making payment: {e}")
        return Response({'error': 'An error occurred while processing the payment.'}, status=status.HTTP_400_BAD_REQUEST)

